/* ------------------------------------------------------------------------
System       : cc65 cross-development environment
Platform     : Apple IIgs or Apple IIe with VOC or CB

Bill Buckels
July 2014

Licence      : You may use this program for whatever you wish as long
			   as you agree that Bill Buckels has no warranty or
			   liability obligations whatsoever from said use.
------------------------------------------------------------------------ */
#include "dhgr.h"
#include "sprites.h"


unsigned char auxbuf[40],mainbuf[40];

/* clear the DHGR screen to color */
void clear140(unsigned char color)
{

	unsigned xoff, x, y;
	unsigned src1 = (unsigned) &auxbuf[0], src2 = src1+39;


	xoff = HB[y];

    for (x = 0,xoff=0; x < 20; x++) {
		auxbuf[xoff]  = dhrbytes[color][0];
		mainbuf[xoff] = dhrbytes[color][1]; xoff++;
		auxbuf[xoff]  = dhrbytes[color][2];
		mainbuf[xoff] = dhrbytes[color][3]; xoff++;
	}

    /* wipe-down 192 scan lines with color */
    for (y=0;y<192;y++) {
#if DHGR_SCREEN_ADDRESS == 0x4000
		xoff = HB[y] + 0x2000;
#else
		xoff = HB[y];
#endif
		maintoaux(src1,src2,xoff);
		memcpy((char *)xoff,(char *)&mainbuf[0],40);
	}
}

/* simple putimage function for this demo */
/* does not preserve the background */
#pragma optimize (push,off)
void PutFragment(unsigned char *PixelData, unsigned width, unsigned height,
                               unsigned x, unsigned y)
{
    unsigned *src  = (unsigned *)0x3c;
    unsigned *dest = (unsigned *)0x42;
	unsigned y2 = y + height, offset = 0, packet = width/2;
    unsigned mainoffset = packet, destoffset;

    /* rasters are split between auxiliary memory and main memory */
	while (y < y2) {
		src[0] = (unsigned) &PixelData[offset];
		src[1] = (unsigned) &PixelData[mainoffset-1];
#if DHGR_SCREEN_ADDRESS == 0x4000
		dest[0] = destoffset = (HB[y] + 0x2000 + (x/2));
#else
		dest[0] = destoffset = (HB[y] + (x/2));
#endif
		/* auxiliary memory */
		asm("sec");
		asm("jsr $c311");
		/* main memory */
		memcpy((char *)destoffset,&PixelData[mainoffset],packet);

		offset+= width;
		mainoffset+=width;
		y++;
	}
}
#pragma optimize (pop)

/* bounce demo */
unsigned char FragDemo(unsigned char *rPixelData, unsigned char *lPixelData,
                       unsigned width, unsigned height, unsigned char BackgroundColor)
{

	int i,j, dx = 4, dy = 1;
	unsigned x1 = 0, x2 = (80 - width);
	unsigned y1 = 0, y2 = (192 - height);
    unsigned char ch = (unsigned char)(demospeed + 48);

    clear140(BackgroundColor);

    while (kbhit() > 0)cgetc();

 do {

  	if (dx > 0)PutFragment((unsigned char *)&rPixelData[0],width,height,x1,y1);
    else PutFragment((unsigned char *)&lPixelData[0],width,height,x1,y1);

    if (demospeed == 0) {
		while (kbhit() == 0);
	}
	else {
	    for (i = 0; i < (demospeed-1); i++)
	    	for (j=0;j<384;j++);
	}

    if ( y1 == y2) dy = -1;
    else if (y1 <1) dy = 1;
    if ( x1 == x2) {
		dx = -4;
		PutFragment((unsigned char *)&lPixelData[0],width,height,x1,y1);
	}
    else if (x1 < 1) {
		dx = 4;
		PutFragment((unsigned char *)&rPixelData[0],width,height,x1,y1);
	}

    x1 += dx;
    y1 += dy;

    if (kbhit() != 0) {
		ch = cgetc();
		if (ch == 13 || ch == 27) break;
		if (ch > 47 && ch < 58) demospeed = (ch - 48);
		while (kbhit() > 0)cgetc();
	}

 } while (ch != 27 || ch != 13);

 while (kbhit() > 0)cgetc();
 return ch;

}